package org.openjdk.gcbench.roots;

import org.openjdk.jmh.annotations.*;

import java.util.concurrent.TimeUnit;
import java.util.concurrent.locks.ReentrantLock;

@Warmup(iterations = 5, time = 1, timeUnit = TimeUnit.SECONDS)
@Measurement(iterations = 5, time = 1, timeUnit = TimeUnit.SECONDS)
@Fork(1)
@BenchmarkMode(Mode.AverageTime)
@OutputTimeUnit(TimeUnit.MILLISECONDS)
@Threads(Threads.MAX)
@State(Scope.Benchmark)
public class NewSyncReentrant {

    SyncPair[] pairs;

    @Param({"100000"})
    int size;

    @Setup
    public void setup() {
        pairs = new SyncPair[size];
        for (int c = 0; c < size; c++) {
            pairs[c] = new SyncPair();
        }
    }

    @TearDown(Level.Iteration)
    public void gc() {
        System.gc();
    }

    @Benchmark
    public void test() throws InterruptedException {
        for (SyncPair pair : pairs) {
            pair.move();
        }
    }

    static class SyncPair {
        final ReentrantLock lock = new ReentrantLock();
        int x, y;

        public void move() {
            lock.lock();
            try {
                x++;
                y--;
            } finally {
                lock.unlock();
            }
        }
    }

}
